% back out the G function using the results from KKS

regression_parameter

b_x=0.0223; % the proposed steady state level of debt
data_b=b_x;
options=optimset('Display','off','LargeScale','off','MaxIter',50,'TolFun',1e-30,'TolX',1e-20);
tau=fsolve('fun_tau',0.4,options);
h=(w*(1-tau)/X)^(1/xi);
e=1/xi*tau/(1-tau);
A=(1-tau)*w*h+F-X/(1+xi)*h^(1+xi)+F/R;
g=(1-e)*A/kap_T;

% first-order differentiation
d_G=kap_G;
d_e=1/xi*1/(1-tau)^2;
d_h=-1/xi*(w/X)^(1/xi)*(1-tau)^(1/xi-1);
d_A=-w*h;
d_T=kap_T*d_G/(-d_e*A+(1-e)*d_A);
d_B=d_G+R-d_T*w*h*(1-e);

% second-order differentiation
dd_G=(kap_G*d_G*d_B-d_G^2)/(d_B*g);
dd_e=1/xi*2/(1-tau)^3;
dd_h=1/xi*(w/X)^(1/xi)*(1/xi-1)*(1-tau)^(1/xi-2);
dd_A=-w*d_h;
dd_T=(kap_T*dd_G-(-dd_e*A+2*(-d_e)*d_A+(1-e)*dd_A)*d_T^2)/(-d_e*A+(1-e)*d_A);
dd_B=dd_G-dd_T*w*h*(1-e)-d_T^2*w*d_h*(1-e)-d_T^2*w*h*(-d_e);

% third-order differentiation
ddd_G=(kap_G*dd_G*d_B^2+kap_G*d_G*dd_B-dd_G*dd_B*g-2*dd_G*d_B*d_G-dd_G*d_G)/(d_B^2*g);
ddd_e=1/xi*6/(1-tau)^4;
ddd_A=-w*dd_h;
ddd_T=(kap_T*ddd_G-(-ddd_e*A+3*(-dd_e)*d_A+3*(-d_e)*dd_A+(1-e)*ddd_A)*d_T^3-2*(-dd_e*A+2*(-d_e)*d_A+(1-e)*dd_A)*d_T*dd_T)/(-d_e*A+(1-e)*d_A);
ddd_B=ddd_G-ddd_T*w*h*(1-e)-3*dd_T*d_T*w*d_h*(1-e)-3*dd_T*d_T*w*h*(-d_e)-d_T^3*w*dd_h*(1-e)-2*d_T^3*w*d_h*(-d_e)-d_T^3*w*h*(-dd_e);

% coefficients in the polynomial function of G
a3=ddd_G/6;
a2=(dd_G-6*a3*b_x)/2;
a1=d_G-2*a2*b_x-3*a3*b_x^2;
a0=g-a1*b_x-a2*b_x^2-a3*b_x^3;

t3=ddd_T/6;
t2=(dd_T-6*t3*b_x)/2;
t1=d_T-2*t2*b_x-3*t3*b_x^2;
t0=tau-t1*b_x-t2*b_x^2-t3*b_x^3;

num_grid=50;
options=optimset('Display','off','LargeScale','off','NonlEqnAlgorithm','gn','MaxIter',50,'TolFun',1e-30,'TolX',1e-20);
clear b
for j=1:num_grid
    
    b(j)=b_min+(j-1)*(b_max-b_min)/(num_grid-1); 
    g(j)=a0+a1*b(j)+a2*b(j)^2+a3*b(j)^3;
    tau(j)=t0+t1*b(j)+t2*b(j)^2+t3*b(j)^3;
        
    % labour supply, its derivative and the elasticity of labor supply
    h(j)=(w*(1-tau(j))/X)^(1/xi);
    e(j)=1/xi*tau(j)/(1-tau(j));
        
    % calculate debt in the next period
    b1(j)=g(j)+R*b(j)-w*tau(j)*h(j);
    g1(j)=a0+a1*b1(j)+a2*b1(j)^2+a3*b1(j)^3;
    % derivatives w.r.t. b(t+1)
    d_g1_b(j)=a1+2*a2*b1(j)+3*a3*b1(j)^2;
        
    %%%%%%%%%%%%% 
    % Equations
    %%%%%%%%%%%%%
        
    % life-time wealth
    A(j)=(1-tau(j))*w*h(j)+F-X/(1+xi)*h(j)^(1+xi)+F/R;
        
    % equations
    ff_1(j,1)=A(j)*(1-e(j))-(1+bet)/(1+omi/(1-omi)*the_o/the)*g(j)/the;
    ff_2(j,1)=-bet*the_o/the*d_g1_b(j)/((omi/(1-omi)*the_o/the+1)*g1(j))-1/g(j);

    % policy function by the projection method
    yyy=feval('fun_result',b(j));
    g_proj(j)=yyy(1); tau_proj(j)=yyy(2,1);
    h_proj(j)=(w*(1-tau_proj(j))/X)^(1/xi);
    b1_proj(j)=g_proj(j)+R*b(j)-w*h_proj(j)*tau_proj(j);
    
end
% figure(1)
% subplot(2,1,1)
% plot(b,ff_1)
% title('deviation from the intratemporal FOC')
% subplot(2,1,2)
% plot(b,ff_2)
% title('deviation from the GEE')
figure(1)
subplot(3,1,1)
plot(b,b1_proj,'-','color','b','linewidth',2)
hold on
plot(b,b1,':','color','r','linewidth',2.5)
hold off
box off
axis([0 max(b) 0 max(b)])
title('panel a')
xlabel('debt')
ylabel('next-period debt')
legend('projection','kks')
subplot(3,1,2)
plot(b,g_proj,'-','color','b','linewidth',2)
hold on
plot(b,g,':','color','r','linewidth',2.5)
hold off
box off
axis([0 max(b) 0 0.25])
xlabel('debt')
ylabel('public spending')
legend('projection','kks')
title('panel b')
subplot(3,1,3)
plot(b,tau_proj,'-','color','b','linewidth',2)
hold on
plot(b,tau,':','color','r','linewidth',2.5)
hold off
box off
axis([0 max(b) 0.3 tau_max])
xlabel('debt')
ylabel('tax rate')
legend('projection','kks')
title('panel c')
% close all
% figure(1)
% plot(b,b1_proj,'-','color','r','linewidth',2)
% hold on
% plot(b,b1,':','color','r','linewidth',2)
% hold off
% box off
% % axis([0 max(b) 0 0.1])
% legend('projection','kks')
